Unit uscaledpi;

{$mode objfpc}{$H+}

Interface

Uses
  Forms, Graphics, Controls, ComCtrls;

Procedure HighDPI(FromDPI: integer);
Procedure ScaleDPI(Control: TControl; FromDPI: integer);
Procedure ScaleImageList(ImgList: TImageList; FromDPI: integer);

Implementation

Uses BGRABitmap, BGRABitmapTypes;

Procedure HighDPI(FromDPI: integer);
Var
  i: integer;
Begin
  For i := 0 To Screen.FormCount - 1 Do
  Begin
    ScaleDPI(Screen.Forms[i], FromDPI);
  End;
End;

Procedure ScaleImageList(ImgList: TImageList; FromDPI: integer);
Var
  TempBmp: TBitmap;
  TempBGRA: Array Of TBGRABitmap;
  NewWidth, NewHeight: integer;
  i: integer;

Begin
  If Screen.PixelsPerInch = FromDPI Then
    exit;

  NewWidth := ScaleX(ImgList.Width, FromDPI);
  NewHeight := ScaleY(ImgList.Height, FromDPI);

  setlength(TempBGRA, ImgList.Count);
  TempBmp := TBitmap.Create;
  For i := 0 To ImgList.Count - 1 Do
  Begin
    ImgList.GetBitmap(i, TempBmp);
    TempBGRA[i] := TBGRABitmap.Create(TempBmp);
    TempBGRA[i].ResampleFilter := rfBestQuality;
    If (TempBGRA[i].Width = 0) Or (TempBGRA[i].Height = 0) Then
      continue;
    While (TempBGRA[i].Width < NewWidth) Or (TempBGRA[i].Height < NewHeight) Do
      BGRAReplace(TempBGRA[i], TempBGRA[i].FilterSmartZoom3(moLowSmooth));
    BGRAReplace(TempBGRA[i], TempBGRA[i].Resample(NewWidth, NewHeight));
  End;
  TempBmp.Free;

  ImgList.Clear;
  ImgList.Width := NewWidth;
  ImgList.Height := NewHeight;

  For i := 0 To high(TempBGRA) Do
  Begin
    ImgList.Add(TempBGRA[i].Bitmap, nil);
    TempBGRA[i].Free;
  End;
End;

Procedure ScaleDPI(Control: TControl; FromDPI: integer);
Var
  n: integer;
  WinControl: TWinControl;
  ToolBarControl: TToolBar;
Begin
  If Screen.PixelsPerInch = FromDPI Then
    exit;

  With Control Do
  Begin
    Left := ScaleX(Left, FromDPI);
    Top := ScaleY(Top, FromDPI);
    Width := ScaleX(Width, FromDPI);
    Height := ScaleY(Height, FromDPI);
    Font.Height := ScaleY(Font.GetTextHeight('Hg'), FromDPI);
  End;

  If Control Is TToolBar Then
  Begin
    ToolBarControl := TToolBar(Control);
    With ToolBarControl Do
    Begin
      ButtonWidth := ScaleX(ButtonWidth, FromDPI);
      ButtonHeight := ScaleY(ButtonHeight, FromDPI);
    End;
  End;

  If Control Is TWinControl Then
  Begin
    WinControl := TWinControl(Control);
    If WinControl.ControlCount > 0 Then
    Begin
      For n := 0 To WinControl.ControlCount - 1 Do
      Begin
        If WinControl.Controls[n] Is TControl Then
        Begin
          ScaleDPI(WinControl.Controls[n], FromDPI);
        End;
      End;
    End;
  End;
End;

End.
